# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'manuskript/ui/tools/frequency_ui.ui'
#
# Created by: PyQt5 UI code generator 5.5.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_FrequencyAnalyzer(object):
    def setupUi(self, FrequencyAnalyzer):
        FrequencyAnalyzer.setObjectName("FrequencyAnalyzer")
        FrequencyAnalyzer.resize(733, 453)
        self.horizontalLayout = QtWidgets.QHBoxLayout(FrequencyAnalyzer)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.tabWidget = QtWidgets.QTabWidget(FrequencyAnalyzer)
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.tab)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.splitter = QtWidgets.QSplitter(self.tab)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.layoutWidget = QtWidgets.QWidget(self.splitter)
        self.layoutWidget.setObjectName("layoutWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.layoutWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox = QtWidgets.QGroupBox(self.layoutWidget)
        self.groupBox.setObjectName("groupBox")
        self.formLayout = QtWidgets.QFormLayout(self.groupBox)
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout.setRowWrapPolicy(QtWidgets.QFormLayout.WrapLongRows)
        self.formLayout.setObjectName("formLayout")
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setObjectName("label")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label)
        self.spnWordMin = QtWidgets.QSpinBox(self.groupBox)
        self.spnWordMin.setProperty("value", 1)
        self.spnWordMin.setObjectName("spnWordMin")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.spnWordMin)
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.txtWordExclude = QtWidgets.QPlainTextEdit(self.groupBox)
        self.txtWordExclude.setPlainText("")
        self.txtWordExclude.setObjectName("txtWordExclude")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.txtWordExclude)
        self.verticalLayout.addWidget(self.groupBox)
        self.progressBarWord = QtWidgets.QProgressBar(self.layoutWidget)
        self.progressBarWord.setProperty("value", 0)
        self.progressBarWord.setObjectName("progressBarWord")
        self.verticalLayout.addWidget(self.progressBarWord)
        self.btnAnalyzeWord = QtWidgets.QPushButton(self.layoutWidget)
        self.btnAnalyzeWord.setObjectName("btnAnalyzeWord")
        self.verticalLayout.addWidget(self.btnAnalyzeWord)
        self.tblWord = QtWidgets.QTableView(self.splitter)
        self.tblWord.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tblWord.setSortingEnabled(True)
        self.tblWord.setObjectName("tblWord")
        self.verticalLayout_2.addWidget(self.splitter)
        self.tabWidget.addTab(self.tab, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.tab_2)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_3 = QtWidgets.QLabel(self.tab_2)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_2.addWidget(self.label_3)
        self.spnPhraseMin = QtWidgets.QSpinBox(self.tab_2)
        self.spnPhraseMin.setProperty("value", 2)
        self.spnPhraseMin.setObjectName("spnPhraseMin")
        self.horizontalLayout_2.addWidget(self.spnPhraseMin)
        self.label_4 = QtWidgets.QLabel(self.tab_2)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_2.addWidget(self.label_4)
        self.spnPhraseMax = QtWidgets.QSpinBox(self.tab_2)
        self.spnPhraseMax.setProperty("value", 5)
        self.spnPhraseMax.setObjectName("spnPhraseMax")
        self.horizontalLayout_2.addWidget(self.spnPhraseMax)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.btnAnalyzePhrase = QtWidgets.QPushButton(self.tab_2)
        self.btnAnalyzePhrase.setObjectName("btnAnalyzePhrase")
        self.horizontalLayout_2.addWidget(self.btnAnalyzePhrase)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.tblPhrase = QtWidgets.QTableView(self.tab_2)
        self.tblPhrase.setSortingEnabled(True)
        self.tblPhrase.setObjectName("tblPhrase")
        self.verticalLayout_3.addWidget(self.tblPhrase)
        self.tabWidget.addTab(self.tab_2, "")
        self.horizontalLayout.addWidget(self.tabWidget)

        self.retranslateUi(FrequencyAnalyzer)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(FrequencyAnalyzer)

    def retranslateUi(self, FrequencyAnalyzer):
        _translate = QtCore.QCoreApplication.translate
        FrequencyAnalyzer.setWindowTitle(_translate("FrequencyAnalyzer", "Frequency Analyzer"))
        self.groupBox.setTitle(_translate("FrequencyAnalyzer", "Settings"))
        self.label.setText(_translate("FrequencyAnalyzer", "Minimum size:"))
        self.label_2.setText(_translate("FrequencyAnalyzer", "Exclude words (comma separated):"))
        self.btnAnalyzeWord.setText(_translate("FrequencyAnalyzer", "Analyze"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("FrequencyAnalyzer", "Word frequency"))
        self.label_3.setText(_translate("FrequencyAnalyzer", "Number of words: from"))
        self.label_4.setText(_translate("FrequencyAnalyzer", "to"))
        self.btnAnalyzePhrase.setText(_translate("FrequencyAnalyzer", "Analyze"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), _translate("FrequencyAnalyzer", "Phrase frequency"))

