# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'manuskript/ui/welcome_ui.ui'
#
# Created: Mon Oct 16 10:45:41 2017
#      by: PyQt5 UI code generator 5.2.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_welcome(object):
    def setupUi(self, welcome):
        welcome.setObjectName("welcome")
        welcome.resize(728, 459)
        self.horizontalLayout = QtWidgets.QHBoxLayout(welcome)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.frame_2 = QtWidgets.QFrame(welcome)
        self.frame_2.setMinimumSize(QtCore.QSize(500, 0))
        self.frame_2.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_2.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_2.setObjectName("frame_2")
        self.verticalLayout_32 = QtWidgets.QVBoxLayout(self.frame_2)
        self.verticalLayout_32.setObjectName("verticalLayout_32")
        self.horizontalLayout_23 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_23.setObjectName("horizontalLayout_23")
        self.tree = QtWidgets.QTreeWidget(self.frame_2)
        self.tree.setMaximumSize(QtCore.QSize(150, 16777215))
        self.tree.setRootIsDecorated(False)
        self.tree.setObjectName("tree")
        item_0 = QtWidgets.QTreeWidgetItem(self.tree)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        item_0.setFont(0, font)
        brush = QtGui.QBrush(QtGui.QColor(230, 230, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        item_0.setBackground(0, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 128))
        brush.setStyle(QtCore.Qt.NoBrush)
        item_0.setForeground(0, brush)
        item_1 = QtWidgets.QTreeWidgetItem(item_0)
        item_1 = QtWidgets.QTreeWidgetItem(item_0)
        item_1 = QtWidgets.QTreeWidgetItem(item_0)
        item_1 = QtWidgets.QTreeWidgetItem(item_0)
        item_1 = QtWidgets.QTreeWidgetItem(item_0)
        item_0 = QtWidgets.QTreeWidgetItem(self.tree)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        item_0.setFont(0, font)
        brush = QtGui.QBrush(QtGui.QColor(230, 230, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        item_0.setBackground(0, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 128))
        brush.setStyle(QtCore.Qt.NoBrush)
        item_0.setForeground(0, brush)
        self.tree.header().setVisible(False)
        self.horizontalLayout_23.addWidget(self.tree)
        self.templateLayout = QtWidgets.QVBoxLayout()
        self.templateLayout.setObjectName("templateLayout")
        self.lytTemplate = QtWidgets.QGridLayout()
        self.lytTemplate.setObjectName("lytTemplate")
        self.templateLayout.addLayout(self.lytTemplate)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.templateLayout.addItem(spacerItem)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.lblTotal = QtWidgets.QLabel(self.frame_2)
        self.lblTotal.setText("")
        self.lblTotal.setObjectName("lblTotal")
        self.horizontalLayout_2.addWidget(self.lblTotal)
        self.btnAddLevel = QtWidgets.QPushButton(self.frame_2)
        self.btnAddLevel.setEnabled(False)
        icon = QtGui.QIcon.fromTheme("list-add")
        self.btnAddLevel.setIcon(icon)
        self.btnAddLevel.setObjectName("btnAddLevel")
        self.horizontalLayout_2.addWidget(self.btnAddLevel)
        self.btnAddWC = QtWidgets.QPushButton(self.frame_2)
        self.btnAddWC.setEnabled(False)
        icon = QtGui.QIcon.fromTheme("applications-debugging")
        self.btnAddWC.setIcon(icon)
        self.btnAddWC.setObjectName("btnAddWC")
        self.horizontalLayout_2.addWidget(self.btnAddWC)
        self.templateLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_23.addLayout(self.templateLayout)
        self.verticalLayout_32.addLayout(self.horizontalLayout_23)
        self.line_4 = QtWidgets.QFrame(self.frame_2)
        self.line_4.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_4.setObjectName("line_4")
        self.verticalLayout_32.addWidget(self.line_4)
        self.chkLoadLastProject = QtWidgets.QCheckBox(self.frame_2)
        self.chkLoadLastProject.setChecked(True)
        self.chkLoadLastProject.setObjectName("chkLoadLastProject")
        self.verticalLayout_32.addWidget(self.chkLoadLastProject)
        self.horizontalLayout_20 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_20.setObjectName("horizontalLayout_20")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_20.addItem(spacerItem1)
        self.btnOpen = QtWidgets.QPushButton(self.frame_2)
        icon = QtGui.QIcon.fromTheme("document-open")
        self.btnOpen.setIcon(icon)
        self.btnOpen.setObjectName("btnOpen")
        self.horizontalLayout_20.addWidget(self.btnOpen)
        self.btnRecent = QtWidgets.QPushButton(self.frame_2)
        icon = QtGui.QIcon.fromTheme("folder-recent")
        self.btnRecent.setIcon(icon)
        self.btnRecent.setObjectName("btnRecent")
        self.horizontalLayout_20.addWidget(self.btnRecent)
        self.btnCreate = QtWidgets.QPushButton(self.frame_2)
        icon = QtGui.QIcon.fromTheme("dialog-ok")
        self.btnCreate.setIcon(icon)
        self.btnCreate.setObjectName("btnCreate")
        self.horizontalLayout_20.addWidget(self.btnCreate)
        self.verticalLayout_32.addLayout(self.horizontalLayout_20)
        self.horizontalLayout.addWidget(self.frame_2)

        self.retranslateUi(welcome)
        QtCore.QMetaObject.connectSlotsByName(welcome)

    def retranslateUi(self, welcome):
        _translate = QtCore.QCoreApplication.translate
        welcome.setWindowTitle(_translate("welcome", "Form"))
        self.tree.headerItem().setText(0, _translate("welcome", "1"))
        __sortingEnabled = self.tree.isSortingEnabled()
        self.tree.setSortingEnabled(False)
        self.tree.topLevelItem(0).setText(0, _translate("welcome", "Templates"))
        self.tree.topLevelItem(0).child(0).setText(0, _translate("welcome", "Empty"))
        self.tree.topLevelItem(0).child(1).setText(0, _translate("welcome", "Novel"))
        self.tree.topLevelItem(0).child(2).setText(0, _translate("welcome", "Novella"))
        self.tree.topLevelItem(0).child(3).setText(0, _translate("welcome", "Short Story"))
        self.tree.topLevelItem(0).child(4).setText(0, _translate("welcome", "Research paper"))
        self.tree.topLevelItem(1).setText(0, _translate("welcome", "Demo projects"))
        self.tree.setSortingEnabled(__sortingEnabled)
        self.btnAddLevel.setText(_translate("welcome", "Add level"))
        self.btnAddWC.setText(_translate("welcome", "Add word count"))
        self.chkLoadLastProject.setText(_translate("welcome", "Next time, automatically open last project"))
        self.btnOpen.setText(_translate("welcome", "Open..."))
        self.btnRecent.setText(_translate("welcome", "Recent"))
        self.btnCreate.setText(_translate("welcome", "Create"))

